//
// If computer is locked or screensaver is running, this plugin pauses the music.
//

#include "stdafx.h"
#include "frontend.h"
#include "resource.h"
#include <mmsystem.h>
#include <stdlib.h>
#pragma comment(lib, "winmm.lib")

#define GPPHDR_VER	0x10
#define APP_NAME "Lock and pause"
#define APP_VERSION " v1.0"

typedef struct {
	int  version;
	char *description;
	int  (*init)(void);
	void (*config)(void);
	void (*quit)(void);
	HWND hwndParent;
	HINSTANCE hDllInstance;
} winampGeneralPurposePlugin;

static int  init(void);
static void config(void);
static void quit(void);
void config_read();

static winampGeneralPurposePlugin gpp = 
{
	GPPHDR_VER,
	APP_NAME APP_VERSION,
	init,
	config,
	quit,
};

char szAppName[] = APP_NAME;
HANDLE hThread;
CRITICAL_SECTION stop_cs;
int stop = 0;
int I_push = 0;
DWORD id;
int change = 0;
WNDPROC DefAmpProc;
DWORD win_version;

int is_lock;
int is_screensaver;
int is_mute;
int check_time;

#ifdef __cplusplus
extern "C" {
#endif
	__declspec( dllexport ) winampGeneralPurposePlugin *winampGetGeneralPurposePlugin(void)
	{
		return &gpp;
	}
#ifdef __cplusplus
}
#endif

BOOL APIENTRY DllMain( HANDLE hModule, 
					  DWORD  ul_reason_for_call, 
					  LPVOID lpReserved
					  )
{
	return TRUE;
}

/** Mixer **/
DWORD		g_dwMuteControlID=-1;
HMIXER		g_hMixer=NULL;
MIXERCAPS	g_mxcaps;

BOOL InitMixer(void)
{
	if(g_hMixer) return TRUE;
	if(mixerOpen(&g_hMixer,0,0,0,MIXER_OBJECTF_MIXER)!=MMSYSERR_NOERROR) return FALSE;
	if(mixerGetDevCaps((UINT)g_hMixer, &g_mxcaps, sizeof(MIXERCAPS))!=MMSYSERR_NOERROR) return FALSE;
	return	TRUE;
}

void ReleaseMixer(void)
{
	if(g_hMixer) mixerClose(g_hMixer);
	g_hMixer=NULL;
	//	g_dwVolumeControlID=-1;
	g_dwMuteControlID=-1;
}

BOOL InitMasterMuteControl()
{
	MIXERLINE mxl;
	MIXERCONTROL mxc;
	MIXERLINECONTROLS mxlc;

	if(g_dwMuteControlID != -1) return TRUE;
	if(InitMixer() == FALSE) return FALSE;

	mxl.cbStruct = sizeof(MIXERLINE);
	mxl.dwComponentType = MIXERLINE_COMPONENTTYPE_DST_SPEAKERS;
	if(mixerGetLineInfo((HMIXEROBJ)g_hMixer,&mxl,
		MIXER_OBJECTF_HMIXER | MIXER_GETLINEINFOF_COMPONENTTYPE)!=MMSYSERR_NOERROR) return FALSE;

	mxlc.cbStruct = sizeof(MIXERLINECONTROLS);
	mxlc.dwLineID = mxl.dwLineID;
	mxlc.dwControlType = MIXERCONTROL_CONTROLTYPE_MUTE;
	mxlc.cControls = 1;
	mxlc.cbmxctrl = sizeof(MIXERCONTROL);
	mxlc.pamxctrl = &mxc;
	if(mixerGetLineControls((HMIXEROBJ)g_hMixer,&mxlc,
		MIXER_OBJECTF_HMIXER | MIXER_GETLINECONTROLSF_ONEBYTYPE) != MMSYSERR_NOERROR) return FALSE;

	g_dwMuteControlID = mxc.dwControlID;

	return TRUE;
}

BOOL GetMasterMute()
{
	MIXERCONTROLDETAILS_BOOLEAN mxcdMute;
	MIXERCONTROLDETAILS mxcd;

	if(InitMasterMuteControl() == FALSE) return FALSE;

	mxcd.cbStruct = sizeof(MIXERCONTROLDETAILS);
	mxcd.dwControlID = g_dwMuteControlID;
	mxcd.cChannels = 1;
	mxcd.cMultipleItems = 0;
	mxcd.cbDetails = sizeof(MIXERCONTROLDETAILS_BOOLEAN);
	mxcd.paDetails = &mxcdMute;
	if (mixerGetControlDetails((HMIXEROBJ)g_hMixer,&mxcd,
		MIXER_OBJECTF_HMIXER | MIXER_GETCONTROLDETAILSF_VALUE)!=MMSYSERR_NOERROR) return FALSE;
	return mxcdMute.fValue;
}

BOOL SetMasterMute(BOOL Val)
{
    MIXERCONTROLDETAILS_BOOLEAN mxcdMute;
    MIXERCONTROLDETAILS mxcd;

    if(InitMasterMuteControl() == FALSE) return FALSE;

    mxcdMute.fValue=Val;

    mxcd.cbStruct = sizeof(MIXERCONTROLDETAILS);
    mxcd.dwControlID = g_dwMuteControlID;
    mxcd.cChannels = 1;
    mxcd.cMultipleItems = 0;
    mxcd.cbDetails = sizeof(MIXERCONTROLDETAILS_BOOLEAN);
    mxcd.paDetails = &mxcdMute;
    if (mixerSetControlDetails((HMIXEROBJ)g_hMixer,&mxcd,
                 MIXER_OBJECTF_HMIXER | MIXER_SETCONTROLDETAILSF_VALUE)!=MMSYSERR_NOERROR) return FALSE;
    
    return TRUE;
}

/** END Mixer **/

void Pause()
{
	I_push = 1;
	SendMessage(gpp.hwndParent,WM_COMMAND,WINAMP_BUTTON3,0);
	I_push = 0;
}

void StartLock()
{
	if (change == 0) {
		Pause();
		change = 1;
	}
}

void StopLock()
{
	if (change == 1) {
		change = 0;
		Pause();
	}
}

BOOL CheckLock()
{
	BOOL lock_b = 0;

	// check screen saver
	if (is_screensaver) {
		if (win_version >= 5) {
			SystemParametersInfo(SPI_GETSCREENSAVERRUNNING, 0, &lock_b, 0);
		}else{
			HDESK ss = OpenDesktop("screen-saver", 0, FALSE, DESKTOP_ENUMERATE);
			if (ss != NULL) {
				lock_b = 1;
				CloseDesktop(ss);
			}
		}
	}

	// check computer lock
	if (is_lock) {
		HDESK hdk = OpenDesktop("Default", 0, FALSE, DESKTOP_SWITCHDESKTOP);
		if (hdk != 0) {
			if (SwitchDesktop(hdk) == 0) {
				lock_b = 1;
			}
		}
		CloseDesktop(hdk);
	}

	// check mute
	if (is_mute) {
		if (GetMasterMute()) {
			lock_b = 1;
		}
	}

	return lock_b;
}

DWORD WINAPI ThreadProc(LPVOID lpV)
{
	int i;

	while(1) {
		int isplay = SendMessage(gpp.hwndParent,WM_WA_IPC,0,IPC_ISPLAYING);
		//** if it returns 0, it is not playing.
		//** If it returns 1, it is playing.
		//** if it returns 3, it is paused. 

		if (change || isplay == 1) {
			if (CheckLock()) {
				StartLock();
			}else{
				StopLock();
			}
		}
		for (i = 0; i < check_time / 100; i ++) {
			EnterCriticalSection(&stop_cs);
			if (stop) return 0;
			LeaveCriticalSection(&stop_cs);
			Sleep(100);
		}
	}
}


LRESULT CALLBACK SubClassProc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp) {
	switch(msg) {
		case WM_COMMAND:
			switch (LOWORD(wp)) {
			case WINAMP_BUTTON3: // Pause button
				if (SendMessage(gpp.hwndParent,WM_WA_IPC,0,IPC_ISPLAYING) != 3)
				{ // now pausing
					break;
				}
			case WINAMP_BUTTON2: // Play button
			case WINAMP_BUTTON1: // preb button
			case WINAMP_BUTTON5: // next button
				if (!I_push) {
					if (is_mute && GetMasterMute()) { // now muting
						SetMasterMute(FALSE);
						if (!CheckLock()) change = 0;
					}
				}
				break;
			}
			break;
	}
	return CallWindowProc(DefAmpProc, hwnd, msg, wp, lp);
}

static int init(void)
{
	OSVERSIONINFO ovi;
	ovi.dwOSVersionInfoSize = sizeof(ovi);
	GetVersionEx(&ovi);
	win_version = ovi.dwMajorVersion;
	config_read();

	// subclass
	DefAmpProc = (WNDPROC)GetWindowLong(gpp.hwndParent, GWL_WNDPROC);
	SetWindowLong(gpp.hwndParent, GWL_WNDPROC, (LONG)SubClassProc);

	InitializeCriticalSection(&stop_cs);
	hThread = CreateThread(NULL, 0, ThreadProc, 0, 0, &id);
	return 0;
}

static void quit(void)
{
	EnterCriticalSection(&stop_cs);
	stop = 1;
	LeaveCriticalSection(&stop_cs);
	WaitForSingleObject(hThread, 200);
	DeleteCriticalSection(&stop_cs);
	SetWindowLong(gpp.hwndParent, GWL_WNDPROC, (LONG)DefAmpProc);
	ReleaseMixer();
}



/** config **/

void config_read()
{
	char ini_file[MAX_PATH], *p;
	GetModuleFileName(gpp.hDllInstance,ini_file,sizeof(ini_file));
	p=ini_file+lstrlen(ini_file);
	while (p >= ini_file && *p != '\\') p--;
	if (++p >= ini_file) *p = 0;
	lstrcat(ini_file,"plugin.ini");

	is_lock = 1;
	is_screensaver = 1;
	is_mute = 1;
	check_time = 500;
	is_lock = GetPrivateProfileInt(szAppName,"ISLOCK",is_lock,ini_file);
	is_screensaver = GetPrivateProfileInt(szAppName,"ISSCREENSAVE",is_screensaver,ini_file);
	is_mute = GetPrivateProfileInt(szAppName,"ISMUTE",is_mute,ini_file);
	check_time = GetPrivateProfileInt(szAppName,"CHECKTIME",check_time,ini_file);
}

void config_write()
{
	char ini_file[MAX_PATH], *p;
	char string[32];
	GetModuleFileName(gpp.hDllInstance,ini_file,sizeof(ini_file));
	p=ini_file+lstrlen(ini_file);
	while (p >= ini_file && *p != '\\') p--;
	if (++p >= ini_file) *p = 0;
	lstrcat(ini_file,"plugin.ini");

	wsprintf(string,"%d",is_lock);
	WritePrivateProfileString(szAppName,"ISLOCK",string,ini_file);
	wsprintf(string,"%d",is_screensaver);
	WritePrivateProfileString(szAppName,"ISSCREENSAVE",string,ini_file);
	wsprintf(string,"%d",is_mute);
	WritePrivateProfileString(szAppName,"ISMUTE",string,ini_file);
	wsprintf(string,"%d",check_time);
	WritePrivateProfileString(szAppName,"CHECKTIME",string,ini_file);
}

BOOL CALLBACK DialogProc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp) {
	switch (msg) {
		case WM_INITDIALOG:
			if (is_lock) CheckDlgButton(hwnd, IDC_CHECK1, BST_CHECKED);
			else CheckDlgButton(hwnd, IDC_CHECK1, BST_UNCHECKED);
			if (is_screensaver) CheckDlgButton(hwnd, IDC_CHECK2, BST_CHECKED);
			else CheckDlgButton(hwnd, IDC_CHECK2, BST_UNCHECKED);
			if (is_mute) CheckDlgButton(hwnd, IDC_CHECK3, BST_CHECKED);
			else CheckDlgButton(hwnd, IDC_CHECK3, BST_UNCHECKED);
			{
				char string[32];
				wsprintf(string, "%d", check_time);
				SetWindowText(GetDlgItem(hwnd, IDC_EDIT3), string);
			}
			break;
		case WM_COMMAND:
			switch (LOWORD(wp)) {
			case IDOK:
				EndDialog(hwnd, 1);
				if (IsDlgButtonChecked(hwnd, IDC_CHECK1) == BST_CHECKED) is_lock = 1;
				else is_lock = 0;
				if (IsDlgButtonChecked(hwnd, IDC_CHECK2) == BST_CHECKED) is_screensaver = 1;
				else is_screensaver = 0;
				if (IsDlgButtonChecked(hwnd, IDC_CHECK3) == BST_CHECKED) is_mute = 1;
				else is_mute = 0;
				if (GetWindowTextLength(GetDlgItem(hwnd, IDC_EDIT3)) < 32) {
					char string[32];
					GetWindowText(GetDlgItem(hwnd, IDC_EDIT3), string, 32);
					int i = atoi(string);
					if (i >= 100 && i <= 10000) check_time = i;
				}
				config_write();
			break;
			case IDCANCEL:
				EndDialog(hwnd, 0);
				break;
			}
			break;
	}
	return FALSE;
}

static void config(void)
{
	/*    MessageBox(gpp.hwndParent,
	"If computer is locked or screensaver is running, this plugin pauses the music.",
	"Lock and pause plugin", MB_OK);
	*/
	DialogBox(gpp.hDllInstance, MAKEINTRESOURCE(IDD_DIALOG1),
		gpp.hwndParent, DialogProc);
}

/** END config **/

